-------------------------------------------------------------------------
-- DeTrashLoot - Delete or sell items in bags based on their market value
-------------------------------------------------------------------------
-- Modules/GUI_debuglog_fmt_loot.lua - Display Loot Data
--
-- Author: Expelliarm5s / June 2022 / All Rights Reserved
--
-- Version 1.1.7
-------------------------------------------------------------------------
-- luacheck: ignore 212 globals DLAPI

local addonName, addon = ...
local GUI = addon:GetModule("GUI")
local L = LibStub("AceLocale-3.0"):GetLocale(addonName)
local LibQTip = LibStub("LibQTip-1.0")
local private = {}
------------------------------------------------------------------------------

------------------------------------------------------------------------------
-- Settings

-- Loot Debug Log Format
private.loot = {}
private.loot.colNames = { L["#"], " " .. L["Class"], L["Cnt"], " " .. L["Item"], L["SellVal"], L["MarketVal"], " " .. L["Action"], "", }
private.temp = 1 -         0.05  -0.1                -0.06                       -0.15         -0.15          -0.13               -0.001
private.loot.colWidth = { -0.05,  0.1,               -0.06,    private.temp,     -0.15,        -0.15,          0.13,               0.001, }
private.loot.colFlex = { "flex", "flex",             "flex",   "search",         "flex",      "flex",         "flex",              "flex", }
private.loot.statusText = {
			L["Sort by No"],
			L["Sort by Class"],
			L["Sort by Count"],
			L["Sort by Item"],
			L["Sort by Sell Value"],
			L["Sort by Market Value"],
			L["Sort by Action"],
			L["Sort by Message"],
		}
private.loot.GetSTData = function(...) return private.GetSTData_loot(...) end

------------------------------------------------------------------------------
-- Scrolling Table Data

-- Return Scrolling Table Data
function private.GetSTData_loot(a, flex, filter)
	GUI:DebugPrintf("GetSTData_loot(%s, %s, %s)", tostring(a), tostring(flex), tostring(filter))

	-- generate content only if visible
	if not GUI.display or GUI.view ~= a then
		GUI:DebugPrintf("... nothing done.")
		return
	end

	local content = {}

	-- generate content only if log data present
	addon.debuglog = addon.debuglog or {}
	if not addon.debuglog[a] then
		return content, flex
	end

	-- generate content
	local colHasContent = {}
	local k = 1
	for _, rdata in pairs(addon.debuglog[a]) do
		local filterMatch = nil  -- = nil -> no filter input = true -> filtered

		local c1, c2, c3, c4, c5, c6, c7, c8 = strmatch(rdata.m or "", "1=(.*)# 2=(.*)# 3=(.*)# 4=(.*)# 5=(.*)# 6=(.*)# 7=(.*)# 8=(.*)#")

		local data = {}
		data[1] = c1 or "" -- Quantity
		data[2] = c2 or "" -- Class
		data[3] = c3 or "" -- SubClass
		data[4] = c4 or "" -- ItemLink
		data[5] = c5 or "" -- SellValue
		data[6] = c6 or "" -- Action
		data[7] = c7 or "" -- State
		data[8] = c8 or "" -- MarketValue
		data[9] = rdata.m or ""

		-- check dropdown
		for j, col in ipairs(flex) do
			colHasContent[j] = colHasContent[j] or data[j]
			if col == "drop" then
				if filter and filter[j] and filter[j] ~= "" then
					if (filterMatch == nil or filterMatch) then
						if data[j] and filter[j] == data[j] then
							filterMatch = true
						else
							filterMatch = false
						end
					end
				end
			end
		end

		-- check search
		for j, col in ipairs(flex) do
			if col == "search" then
				if filter and filter[j] and filter[j] ~= "" then
					if (filterMatch == nil or filterMatch) then
						-- search in all column data
						local found = false
						for m, _ in pairs(data) do
							if data[m] and type(data[m]) == "string" and strfind(strlower(data[m]), strlower(filter[j])) then
								filterMatch = true
								found = true
							end
						 end
						if not found then
							filterMatch = false
						end
					end
				end
			end
		end

		if filterMatch or filterMatch == nil then
			-- convert data into final widget data
			local en = {}
			en.data = {}

			local q = "ffffff"
			if tonumber(data[7]) and addon.Loot.COLOR[tonumber(data[7])] then
				q = addon.Loot.COLOR[tonumber(data[7])]
			end

			local class = format("%02d", tonumber(data[2]) or 0) .. format("%02d", tonumber(data[3]) or 0)

			en.data[1] = {"|cff" .. q .. tostring(k) .. "|r", k}
			en.data[2] = {"|cff" .. q .. class .. "|r", class}
			private.colClassSubClass = 2
			en.data[3] = {"|cff" .. q .. (data[1] or "") .. "|r", tonumber(data[1]) or 0}

			local link = data[4]
			if link and addon.Item:GetItemIsBOP(link) then
				link = link .. ITEM_QUALITY_COLORS[8].hex .. " BOP" .. FONT_COLOR_CODE_CLOSE
			end
			en.data[4] = {link, addon.Item:GetItemName(data[4]) or ""}
			private.colLinkC = 4
			en.data[5] = {addon.Item:FromCopper(tonumber(data[5])) or "", tonumber(data[5]) or 0}
			en.data[6] = {addon.Item:FromCopper(tonumber(data[8])) or "", tonumber(data[8]) or 0}
			-- GUI.colMarketValue = 6
			en.data[7] = {"|cff" .. q .. (data[6] or "") .. "|r", data[6] or ""}
			private.colAction = 7

			if addon.isDebug then
				en.data[8] = {data[9] or "", data[9] or ""}
			else
				en.data[8] = {"", ""}
			end

			en.data[9] = {addon.Loot.STATE[tonumber(data[7]) or 0] or "", tonumber(data[7]) or 0}
			private.colState = 9

			en.data[10] = {tonumber(data[2]), tonumber(data[2]) or 0}
			private.colClass = 10
			en.data[11] = {tonumber(data[3]), tonumber(data[3]) or 0}
			private.colSubClass = 11
			en.data[12] = {data[4], addon.Item:GetItemName(data[4]) or ""}
			private.colLink = 12

			tinsert(content, en)
			k = k + 1
		end
	end

	-- disable dropdown if not present
	for j, col in ipairs(flex) do
		if col == "drop" and not colHasContent[j] then
			flex[j] = ""
		end
	end

	return content, flex
end

-- Provide left + right click on table frame
GUI.debugLog_OnClick = GUI.debugLog_OnClick or {}
function GUI.debugLog_OnClick.loot(btn, data, button)
	GUI:DebugPrintf("OnClick: %s %s %s ", tostring(btn), tostring(data), tostring(button))
	if button == "RightButton" then
		-- Blacklist
		if data and data.data and data.data[private.colLink] and data.data[private.colLink][1] then
			-- addon:Printf(addon.Item:EscapedLink(data.data[private.colLink][1]))
			addon.Loot:SetState(data.data[private.colLink][1], addon.Loot.BLACK)
			addon.Loot:FillLogFromBags(addon.GUI.view)
			addon.GUI.DebugLog_UpdateData(addon.GUI.view, true)
		end
	end
	if button == "LeftButton" then
		if data and data.data and data.data[private.colLink] and data.data[private.colLink][1] then
			-- addon:Printf(addon.Item:EscapedLink(data.data[private.colLink][1]))
			if IsShiftKeyDown() then
				addon.Loot:SetState(data.data[private.colLink][1])
			else
				addon.Loot:SetState(data.data[private.colLink][1], addon.Loot.WHITE)
			end
			addon.Loot:FillLogFromBags(addon.GUI.view)
			addon.GUI.DebugLog_UpdateData(addon.GUI.view, true)
		end
	end
end

-- Provide tooltip on content button
GUI.debugLog_OnEnter = GUI.debugLog_OnEnter or {}
function GUI.debugLog_OnEnter.loot(btn, data)
	GUI:DebugPrintf("OnEnter: %s %s %s ", tostring(btn), tostring(data))

	GUI:SetStatusLine(L["Click to whitelist / RightClick to blacklist / ShiftClick to unlist"])

	if btn.btnCol and btn.btnCol == private.colLinkC then
		local link
		if data and data.data and data.data[private.colLink] then
			link = data.data[private.colLink][1]
		end

		if link then
			GameTooltip:SetOwner(btn, "ANCHOR_TOPRIGHT")
			pcall(GameTooltip.SetHyperlink, GameTooltip, link)
		end
	elseif btn.btnCol and btn.btnCol == private.colAction then
		local tooltip = LibQTip:Acquire(addon)
		if tooltip then
			tooltip:ClearAllPoints()
			tooltip:SetClampedToScreen(true)
			tooltip:SetPoint("BOTTOMRIGHT", btn, 150, 0)
			tooltip:SetAutoHideDelay(0.1, btn)
			tooltip:EnableMouse(true)
			tooltip:Clear()
			tooltip:SetColumnLayout(3,"LEFT", "RIGHT", "RIGHT")

			local q = "ffffff"
			if tonumber(data.data[private.colState][2]) and addon.Loot.COLOR[tonumber(data.data[private.colState][2])] then
				q = addon.Loot.COLOR[tonumber(data.data[private.colState][2])]
			end

			local line = tooltip:AddLine()
			tooltip:SetCell(line, 1, "|cfffed100" .. L["State:"] .. "|r  ")
			tooltip:SetCell(line, 3, "|cff" .. q .. (data.data[private.colState][1] or "") .. "|r")
			tooltip:AddSeparator()

			local numClass = tonumber(data.data[private.colClass][1])
			if numClass and addon.Loot.CLASS[numClass] then
				line = tooltip:AddLine()
				tooltip:SetCell(line, 1, "|cfffed100" .. L["Item Class:"] .. "|r  ")
				tooltip:SetCell(line, 2, "|cff" .. q .. tostring(numClass or "") .. "|r")
				local class = addon.Loot.CLASS[numClass].NAME
				tooltip:SetCell(line, 3, "|cff" .. q .. class .. "|r")
			end

			local numSubClass = tonumber(data.data[private.colSubClass][1])
			if numClass and numSubClass and addon.Loot.CLASS[numClass] and addon.Loot.CLASS[numClass].SUB then
				line = tooltip:AddLine()
				tooltip:SetCell(line, 1, "|cfffed100" .. L["Item SubClass:"] .. "|r  ")
				tooltip:SetCell(line, 2, "|cff" .. q .. tostring(numSubClass or "") .. "|r")
				local subClass = addon.Loot.CLASS[numClass].SUB[numSubClass + 1] or ""
				tooltip:SetCell(line, 3, "|cff" .. q .. subClass .. "|r")
			end

			tooltip:Show()
		end
	elseif btn.btnCol and btn.btnCol == private.colClassSubClass then
		local tooltip = LibQTip:Acquire(addon)
		if tooltip then
			tooltip:ClearAllPoints()
			tooltip:SetClampedToScreen(true)
			tooltip:SetPoint("BOTTOMRIGHT", btn, 150, 0)
			tooltip:SetAutoHideDelay(0.1, btn)
			tooltip:EnableMouse(true)
			tooltip:Clear()
			tooltip:SetColumnLayout(3,"LEFT", "RIGHT", "RIGHT")

			local q = "ffffff"
			if tonumber(data.data[private.colState][2]) and addon.Loot.COLOR[tonumber(data.data[private.colState][2])] then
				q = addon.Loot.COLOR[tonumber(data.data[private.colState][2])]
			end

			local line = tooltip:AddLine()
			tooltip:SetCell(line, 1, "|cfffed100" .. L["Class:"] .. "|r  ")
			local numClass = tonumber(data.data[private.colClass][1])
			tooltip:SetCell(line, 2, "|cff" .. q .. tostring(numClass or "") .. "|r")
			local class = ""
			if numClass and addon.Loot.CLASS[numClass] then
				class = addon.Loot.CLASS[numClass].NAME
			end
			tooltip:SetCell(line, 3, "|cff" .. q .. class .. "|r")

			line = tooltip:AddLine()
			tooltip:SetCell(line, 1, "|cfffed100" .. L["SubClass:"] .. "|r  ")
			local numSubClass = tonumber(data.data[private.colSubClass][1])
			tooltip:SetCell(line, 2, "|cff" .. q .. tostring(numSubClass or "") .. "|r")
			local subClass = ""
			if numSubClass and numClass and addon.Loot.CLASS[numClass] and addon.Loot.CLASS[numClass] and addon.Loot.CLASS[numClass].SUB then
				subClass = addon.Loot.CLASS[numClass].SUB[numSubClass + 1] or ""
			end
			tooltip:SetCell(line, 3, "|cff" .. q .. subClass .. "|r")

			tooltip:Show()
		end
	end
end

-- Hide Tooltip
GUI.debugLog_OnLeave = GUI.debugLog_OnLeave or {}
function GUI.debugLog_OnLeave.loot(btn, data)
	GUI:DebugPrintf("OnLeave: %s %s %s ", tostring(btn), tostring(data))

	GUI:SetStatusLine()

	if btn.btnCol and btn.btnCol == private.colLinkC then
		GameTooltip:Hide()
	end
end

------------------------------------------------------------------------------
-- Create Loot Tab
function GUI.CreateLootTab(a, additionalUICB, forced)
	GUI:DebugPrintf("CreateLootTab(%s)", tostring(a))

	if not a or type(a) ~= "string" then
		-- should not never happen!
		GUI:DebugPrintf("ERR~  missing tab name!")
		return
	end

	if GUI.guiTabs.select[a] and not forced then
		GUI:DebugPrintf("Tab %s exists.", tostring(a))
		return
	end

	if not addon.db.global.extendedUI then
		private.loot.colNames = { L["#"], " " .. L["Class"], L["Cnt"], " " .. L["Item"], L["SellVal"], L["MarketVal"], " " .. L["Action"], "", }
		private.temp = 1 -         0.001 -0.001              -0.06                       -0.15         -0.15          -0.13               -0.001
		private.loot.colWidth = { -0.001, 0.001,             -0.06,    private.temp,     -0.15,        -0.15,          0.13,               0.001, }
	else
		private.loot.colNames = { L["#"], " " .. L["Class"], L["Cnt"], " " .. L["Item"], L["SellVal"], L["MarketVal"], " " .. L["Action"], "", }
		private.temp = 1 -         0.05  -0.1                -0.06                       -0.15         -0.15          -0.13               -0.001
		private.loot.colWidth = { -0.05,  0.1,               -0.06,    private.temp,     -0.15,        -0.15,          0.13,               0.001, }
	end

	-- create a new log format
	if not GUI.DebugLog_IsFormatRegistered("loot") or forced then
		GUI.DebugLog_RegisterFormat("loot", private.loot)
	end

	-- adjust default data widget parameters
	addon.db.global.showExport = false
	GUI.debugLog_defaultNoSelfTab = true
	GUI.debugLog_defaultTextSize = tonumber(addon.db.global.textFontSize) or 11
	GUI.debugLog_defaultHeight = 550
	GUI.debugLog_defaultWidth = 600

	if addon.db.global.marketValueSort then
		GUI.debugLog_defaultSortTitle = -6 -- change this to: market value column number
	else
		GUI.debugLog_defaultSortTitle =  1
	end

	GUI.debugLog_defaultFrameWidget = "DTLFrame"
	GUI.debugLog_defaultTabGroupWidget = "DTLTabGroup"
	GUI.DebugLog_SetWidgets({ data = "DTLData"})

	-- use /dtlsll, no magic button anymore
	-- GUI.debugLog_defaultMagicButtonCB = addon.Loot.MagicButtonCB
	-- GUI.debugLog_defaultMagicButtonText = addon.Loot:MagicButtonText()
	-- GUI.debugLog_defaultMagicButtonStatusText = addon.Loot:MagicButtonStatusText()

	-- create tab as custom debug log
	GUI.DebugLog_CreateTab(a, "loot", nil, additionalUICB)

	-- no need to have a timer
	GUI.guiTabs.timer[a] = nil

	GUI:DebugPrintf("WARN~Log for '" .. a .. "' created")

	-- no implicit Load()
	-- GUI:Load(a)
	GUI:DebugPrintf("  <<CreateLootTab(%s)", tostring(a))
end

-- EOF
