-------------------------------------------------------------------------
-- DeTrashLoot - Delete or sell items in bags based on their market value
-------------------------------------------------------------------------
-- Modules/Loot.lua - Collecting Loot Data
--
-- Author: Expelliarm5s / June 2022 / All Rights Reserved
--
-- Version 1.1.7
-------------------------------------------------------------------------
-- luacheck: ignore 212 globals DLAPI

-- luacheck: globals GetInventoryItemLink UseContainerItem DeleteCursorItem MerchantFrame MerchantFrameCloseButton
-- luacheck: globals LE_ITEM_CLASS_CONSUMABLE LE_ITEM_CLASS_CONTAINER LE_ITEM_CLASS_WEAPON LE_ITEM_CLASS_GEM LE_ITEM_CLASS_ARMOR
-- luacheck: globals LE_ITEM_CLASS_REAGENT LE_ITEM_CLASS_PROJECTILE LE_ITEM_CLASS_TRADEGOODS LE_ITEM_CLASS_ITEM_ENHANCEMENT
-- luacheck: globals LE_ITEM_CLASS_RECIPE LE_ITEM_CLASS_QUIVER LE_ITEM_CLASS_QUESTITEM LE_ITEM_CLASS_KEY LE_ITEM_CLASS_MISCELLANEOUS
-- luacheck: globals LE_ITEM_CLASS_GLYPH LE_ITEM_CLASS_BATTLEPET LE_ITEM_CLASS_WOW_TOKEN LE_ITEM_MISCELLANEOUS_JUNK

local addonName, addon = ...
local Loot = addon:NewModule("Loot", "AceEvent-3.0", "AceConsole-3.0", "AceHook-3.0")
local AceGUI = LibStub("AceGUI-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale(addonName)
local LibQTip = LibStub("LibQTip-1.0")
local private = {}
-------------------------------------------------------------------------------------------------------

private.defaultTab = L["Loot"]

private.maxSellCnt = 11
private.qualityTreshold = Enum.ItemQuality.Epic

Loot.NONE = 0
Loot.WHITE = 1
Loot.BLACK = 2
Loot.HANDLE = 3
Loot.DONT = 4
Loot.TRESH = 5
Loot.IGNORED = 6

Loot.STATE = {
	[Loot.NONE] = L["undefined"],
	[Loot.WHITE] = L["whitelisted"],
	[Loot.BLACK] = L["blacklisted"],
	[Loot.HANDLE] = L["sell/delete"],
	[Loot.DONT] = L["do nothing"],
	[Loot.TRESH] = L["above treshold"],
	[Loot.IGNORED] = L["ignored"],
}

Loot.COLOR = {
	[Loot.NONE] = "ffffff",
	[Loot.WHITE] = "88ff88", -- green
	[Loot.BLACK] = "ff8888", -- red
	[Loot.HANDLE] = "ffffff",
	[Loot.DONT] = "8888ff",
	[Loot.TRESH] = "ff88e0", -- magenta
	[Loot.IGNORED] = "8888ff", -- blue
}

Loot.CLASS = {}
Loot.CLASS[LE_ITEM_CLASS_CONSUMABLE] = {
	NAME = L["Consumable"],
	SUB = {
		L["Explosives and Devices"],
		L["Potion"],
		L["Elixir"],
		L["Flask"],
		L["Scroll (OBSOLETE)"],
		L["Food & Drink"],
		L["Item Enhancement (OBSOLETE)"],
		L["Bandage"],
		L["Other"],
		L["Vantus Runes"],
	}
}

Loot.CLASS[LE_ITEM_CLASS_CONTAINER] = {
	NAME = L["Container"],
}

Loot.CLASS[LE_ITEM_CLASS_WEAPON] = {
	NAME = L["Weapon"],
	SUB = {
		L["One-Handed Axes"],
		L["Two-Handed Axes"],
		L["Bows"],
		L["Guns"],
		L["One-Handed Maces"],
		L["Two-Handed Maces"],
		L["Polearms"],
		L["One-Handed Swords"],
		L["Two-Handed Swords"],
		L["Warglaives"],
		L["Staves"],
		L["Bear Claws"],
		L["CatClaws"],
		L["Fist Weapons"],
		L["Miscellaneous"],
		L["Daggers"],
		L["Thrown Classic"],
		L["Spears"],
		L["Crossbows"],
		L["Wands"],
		L["Fishing Poles"],
	}
}
Loot.CLASS[LE_ITEM_CLASS_GEM] = {
	NAME = L["Gem"],
}
Loot.CLASS[LE_ITEM_CLASS_ARMOR] = {
	NAME = L["Armor"],
}
Loot.CLASS[LE_ITEM_CLASS_REAGENT] = {
	NAME = L["Reagent"],
}
Loot.CLASS[LE_ITEM_CLASS_PROJECTILE] = {
	NAME = L["Projectile"],
}
Loot.CLASS[LE_ITEM_CLASS_TRADEGOODS] = {
	NAME = L["Tradeskill"],
	SUB = {
		L["Trade Goods (OBSOLETE)"],
		L["Parts"],
		L["Explosives (OBSOLETE)"],
		L["Devices (OBSOLETE)"],
		L["Jewelcrafting"],
		L["Cloth"],
		L["Leather"],
		L["Metal & Stone"],
		L["Cooking"],
		L["Herb"],
		L["Elemental"],
		L["Other"],
		L["Enchanting"],
		L["Materials (OBSOLETE)"],
		L["Item Enchantment (OBSOLETE)"],
		L["Weapon Enchantment (OBSOLETE)"],
		L["Inscription"],
		L["Explosives and Devices (OBSOLETE)"],
	}
}
Loot.CLASS[LE_ITEM_CLASS_ITEM_ENHANCEMENT] = {
	NAME = L["Item Enhancement"],
}
Loot.CLASS[LE_ITEM_CLASS_RECIPE] = {
	NAME = L["Recipe"],
}
Loot.CLASS[LE_ITEM_CLASS_QUIVER] = {
	NAME = L["Quiver"],
}
Loot.CLASS[LE_ITEM_CLASS_QUESTITEM] = {
	NAME = L["Questitem"],
}
Loot.CLASS[LE_ITEM_CLASS_KEY] = {
	NAME = L["Key"],
}
Loot.CLASS[LE_ITEM_CLASS_MISCELLANEOUS] = {
	NAME = L["Miscellaneous"],
	SUB = {
		L["Junk"],
		L["Reagent"],
		L["Companion Pets"],
		L["Holliday"],
		L["Other"],
		L["Mount"],
		L["Mount Equipment"],
	}
}
Loot.CLASS[LE_ITEM_CLASS_GLYPH] = {
	NAME = L["Glyph"],
}
Loot.CLASS[LE_ITEM_CLASS_BATTLEPET] = {
	NAME = L["Battle Pet"],
}
Loot.CLASS[LE_ITEM_CLASS_WOW_TOKEN] = {
	NAME = L["WoWToken"],
}

------------------------------------------------------------------------------
-- Debug Stuff

function Loot:DebugPrintf(...)
	if addon.isDebug then
		local status, res = pcall(format, ...)
		if status then
			addon:DebugLog("Loo~" .. res)
		end
	end
end

------------------------------------------------------------------------------
-- Enable Loot

function Loot:Login()
	Loot:DebugPrintf("Login()")

	-- White-/Blacklisted Items
	addon.db.global.listLoot = addon.db.global.listLoot or {}
	Loot.listLoot = addon.db.global.listLoot

	-- are we at a vendor?
	Loot.atVendor = false

	-- we are using the debugLog environment
	addon.debuglog = addon.debuglog or {}

	-- lookup all loot present at login
	Loot.loot, Loot.loginLoot = Loot:ScanLootInBagsAndInventory()

	-- update loot list after looting
	Loot:RegisterEvent("BAG_UPDATE_DELAYED")

	-- change DELETE to SELL at vendor
	Loot:RegisterEvent("MERCHANT_SHOW")
	Loot:RegisterEvent("MERCHANT_CLOSED")

	Loot:DebugPrintf("  <<Login()")
end

function Loot:BAG_UPDATE_DELAYED()
	Loot:DebugPrintf("OK~BAG_UPDATE_DELAYED()")

	-- v1.1.2: less laggy: update once per timer
	private.bagUpdated = true

	-- v1.1.2: more laggy: update after bag change
	--[[
	if addon.GUI.display then
		Loot:FillLogFromBags(addon.GUI.view)
		addon.GUI.DebugLog_UpdateData(addon.GUI.view, true)
	end
	]]
end

function Loot:MERCHANT_SHOW()
	Loot:DebugPrintf("OK~MERCHANT_SHOW()")
	Loot.atVendor = true
	if not addon.GUI.display and addon.db.global.openWithVendor and not private.listStayClosed then
		addon:ShowGUI()
	end
	if addon.GUI.display then
		if private.btnAction then
			private.btnAction:SetText(L["Sell"])
		end
		Loot:FillLogFromBags(addon.GUI.view)
		addon.GUI.DebugLog_UpdateData(addon.GUI.view, true)
	end
	Loot:CreateDTLButton()
end

function Loot:MERCHANT_CLOSED()
	Loot:DebugPrintf("OK~MERCHANT_CLOSED()")
	Loot.atVendor = false
	if addon.GUI.display then
		if addon.db.global.closeWithVendor then
			addon.GUI.display:Fire("OnClose")
		end
	end
	if addon.GUI.display then
		if private.btnAction then
			private.btnAction:SetText(L["Delete"])
		end
		Loot:FillLogFromBags(addon.GUI.view)
		addon.GUI.DebugLog_UpdateData(addon.GUI.view, true)
	end
end

-- v1.1.2: Place the DTL button on the vendor frame
function Loot:CreateDTLButton(list)
	Loot:DebugPrintf("CreateDTLButton(%s)", tostring(list))

	if MerchantFrame and MerchantFrameCloseButton and not private.dtlButton then
		private.dtlButton = CreateFrame("Button", nil, MerchantFrame, "UIPanelButtonTemplate")
		private.dtlButton:SetHeight(20)
		private.dtlButton:RegisterForClicks("LeftButtonUp")
		private.dtlButton:SetText("DTL")

		-- leftclick opens or closes list
		private.dtlButton:SetScript("OnClick", function(_, mouseButton, _)
			Loot:DebugPrintf("CreateDTLButton_OnClick(%s, %s)", tostring(mouseButton), tostring(list))
			if addon.GUI.display then
				addon.GUI.display:Hide()
				private.listStayClosed = true
			else
				private.listStayClosed = false
				addon:ShowGUI()
			end
		end)

		-- mouseover shows usage tooltip
		private.dtlButton:SetScript("OnEnter", function(this)
			local tooltip = LibQTip:Acquire(addon)
			if tooltip then
				tooltip:ClearAllPoints()
				tooltip:SetClampedToScreen(true)
				tooltip:SetPoint("BOTTOMLEFT", this, 0, 20)
				tooltip:SetAutoHideDelay(0.1, this)
				tooltip:EnableMouse(true)
				tooltip:Clear()
				tooltip:SetColumnLayout(2,"LEFT", "RIGHT")

				local line = tooltip:AddHeader()
				tooltip:SetCell(line, 1, "|cfffed100" .. addon.METADATA.NAME .. " (" .. addon.METADATA.VERSION .. ")|r", nil, "CENTER", 2)
				tooltip:AddSeparator()
				line = tooltip:AddLine()
				if addon.GUI.display then
					tooltip:SetCell(line, 1, L["Hide Item List"], 2)
				else
					tooltip:SetCell(line, 1, L["Show Item List"], 2)
				end

				tooltip:Show()
			end
		end)

		-- left of MerchantFrame Close Button
		private.dtlButton:SetPoint("RIGHT", MerchantFrameCloseButton, "LEFT", 6, 20)
		private.dtlButton:SetWidth(private.dtlButton:GetTextWidth() + 45)
	end
end

function Loot:SetState(link, state)
	Loot:DebugPrintf("SetState(%s, %s)", tostring(link), tostring(state))

	if not state then
		state = Loot.NONE
	end

	if not Loot.STATE[state] then
		Loot:DebugPrintf("ERR~  invalid state: %", tostring(state))
		return
	end

	Loot.listLoot = Loot.listLoot or {}
	if link and state then
		if Loot.listLoot[link] and Loot.listLoot[link].state then
			Loot:DebugPrintf("  was %s, now %s", tostring(Loot.STATE[Loot.listLoot[link].state]), tostring(Loot.STATE[state]))
			Loot.listLoot[link].state = state
		else
			Loot:DebugPrintf("  now %s", tostring(Loot.STATE[state]))
			Loot.listLoot[link] = {state = state}
		end
		addon:Printf(L["%s is now %s"], tostring(link), tostring(Loot.STATE[Loot.listLoot[link].state]))

		-- if not white- or blacklisted: no need to list
		if Loot.listLoot[link].state == Loot.NONE then
			Loot.listLoot[link] = nil
		end
	else
		Loot:DebugPrintf("ERR~  no valid link %s or state %", tostring(Loot.listLoot[link]), tostring(state))
	end
end

-- create and return button/other widget
function private.AddButton(widgetType, label, status)
	if not status then
		widgetType, label, status = "Button", widgetType, label
	end
	local button = assert(AceGUI:Create(widgetType))
	if button.SetText then
		button:SetText(tostring(label))
	end
	button:SetCallback("OnEnter", function() addon.GUI:SetStatusLine(status) end)
	button:SetCallback("OnLeave", function() addon.GUI:SetStatusLine("") end)
	return button
end

-- create additional UI elements
function Loot.CreateLootListWidgets(tab, container)
	Loot:DebugPrintf("CreateLootListWidgets(%s, %s)", tostring(tab), tostring(container))

	if not container then
		return
	end

	local thl = private.AddButton("Label", "|cfffed100" .. L["MV Treshold:"] .. "|r","")
	thl:SetRelativeWidth(0.25)
	container:AddChild(thl)

	local gold = (tonumber(addon.db.global.handleTreshold) or 1000000) / 100 / 100 -- default: 100g
	local th = private.AddButton("EditBox", tostring(gold) .. "|cffffd70ag|r", L["Insert treshold value below to sell/delete"])
	th:SetRelativeWidth(0.25)
	th:SetCallback("OnEnterPressed",
		function()
			local value = th:GetText()
			if tonumber(value) then
				addon.db.global.handleTreshold = floor(abs(value)) * 100 * 100
			else
				local nValue = value:match("([0-9]+)s")
				if tonumber(nValue) then
					addon.db.global.handleTreshold = floor(abs(nValue)) * 100
					value = nValue
				else
					nValue = value:match("([0-9]+)")
					if tonumber(nValue) then
						addon.db.global.handleTreshold = floor(abs(nValue)) * 100 * 100
						value = nValue
					end
				end
			end
			local ggold = tonumber(addon.db.global.handleTreshold) / 100 / 100
			if not tonumber(value) then
				addon.db.global.handleTreshold = "5000000"
				ggold = 500
				addon:Printf(L["Invalid treshold value! Reset to 500g!"])
			else
				addon:Printf(L["New treshold: %s|cffffd70ag|r"], ggold)
			end
			th:SetText(tostring(ggold) .. "|cffffd70ag|r")
			AceGUI:ClearFocus()

			Loot:DebugPrintf("WARN~New Treshold: %s", tostring(addon.db.global.handleTreshold))

			Loot:FillLogFromBags(addon.GUI.view)
			addon.GUI.DebugLog_UpdateData(addon.GUI.view, true)
		end
	)
	container:AddChild(th)

	private.btnAction = nil
	if Loot.atVendor then
		private.btnAction = private.AddButton(L["Sell"], L["Selling (at Vendor) or deleting items."])
	else
		private.btnAction = private.AddButton(L["Delete"], L["Selling (at Vendor) or deleting items."])
	end

	private.btnAction:SetRelativeWidth(0.5)
	private.btnAction:SetCallback("OnClick",
		function()
			Loot:DebugPrintf("WARN~Sell/Delete Button")
			Loot:SellOrDelete()
		end
	)
	container:AddChild(private.btnAction)

	local psl = private.AddButton("Label", "|cfffed100" .. L["MV Price Source:"] .. "|r","")
	psl:SetRelativeWidth(0.25)
	container:AddChild(psl)

	private.ps = private.ps or addon.Item:GetPriceSources()
	local pstitel -- L["Price Source"]
	if addon.db.global.priceSource and private.ps[addon.db.global.priceSource] then
		pstitel = private.ps[addon.db.global.priceSource]
	else
		addon:Printf(L["Invalid or unavailable price source: reset to VendorSell!"])
		addon.db.global.priceSource = "VendorSell"
		pstitel = private.ps[addon.db.global.priceSource]
	end

	local psdd = private.AddButton("Dropdown", pstitel, L["Choose a Price Source for Market Value"])
	psdd:SetRelativeWidth(0.75)
	psdd:SetList(private.ps)
	psdd:SetCallback("OnValueChanged",
		function(_, _, key)
			-- addon:Printf("Key = " .. tostring(key))
			if key and private.ps[key] then
				addon.db.global.priceSource = key
				addon:Printf(L["Price Source is now: %s"], private.ps[addon.db.global.priceSource])

				Loot:FillLogFromBags(addon.GUI.view)
				addon.GUI.DebugLog_UpdateData(addon.GUI.view, true)
			end
		end
	)
	container:AddChild(psdd)
end

function Loot:MagicButtonCB()
	if addon.db.global.showLoginLoot then
		addon.db.global.showLoginLoot = false
	else
		addon.db.global.showLoginLoot = true
	end

	addon.GUI.display:SetMagicButtonText(Loot:MagicButtonText())
	addon.GUI.display:SetMagicButtonStatusText(Loot:MagicButtonStatusText())

	-- display changes
	Loot:FillLogFromBags(addon.GUI.view)
	addon.GUI.DebugLog_UpdateData(addon.GUI.view, true)
end

function Loot:MagicButtonText()
	if addon.db.global.showLoginLoot then
		return L["HL"]
	else
		return L["SL"]
	end
end

function Loot:MagicButtonStatusText()
	if addon.db.global.showLoginLoot then
		return L["Hide loot present at login/reload."]
	else
		return L["Show loot present at login/reload. BE CAREFUL!"]
	end
end

function Loot:SetRebuildUI()
	private.rebuildUI = true
end

-- Create Tab, Fill Data, Show UI
function Loot:Load(tab)
	Loot:DebugPrintf("Load(%s)", tostring(tab))

	local theTab = tab or private.defaultTab

	-- create Log/Tab if not present
	if not addon.debuglog[theTab] or private.rebuildUI then
		Loot:DebugPrintf("  no log %s", tostring(theTab))

		-- flag tab for reload
		if private.rebuildUI then
			Loot:DebugPrintf("  rebuildUI for %s: #addon.GUI.guiTabs.order = %s", tostring(theTab), tostring(#addon.GUI.guiTabs.order))
			local thisName
			for i, name in ipairs(addon.GUI.guiTabs.order) do
				Loot:DebugPrintf("  rebuildUI for %s: i=%s, name=%s", tostring(theTab), tostring(i), tostring(name))
				if name and name == theTab then
					thisName = i
				end
			end
			if thisName then
				Loot:DebugPrintf("  rebuildUI for %s: tab num %s", tostring(theTab), tostring(thisName))
				table.remove(addon.GUI.guiTabs.order, thisName)
			else
				Loot:DebugPrintf("ERR~  rebuildUI for %s: no tab num found!", tostring(theTab))
			end
		end

		addon.GUI.CreateLootTab(theTab, Loot.CreateLootListWidgets, private.rebuildUI)
	else
		-- to show this tab in GUI:Load()
		addon.GUI.guiTabs.lastSelected = theTab
		Loot:DebugPrintf("  log %s", tostring(theTab))
	end

	private.rebuildUI = false

	-- put relevant loot into the log
	Loot:FillLogFromBags(theTab)

	-- show the GUI
	Loot:DebugPrintf("  -> GUI:Load(%s)", tostring(theTab))
	addon.GUI:Load(theTab)
	Loot:DebugPrintf("  <<Load(%s)", tostring(theTab))
end

------------------------------------------------------------------------------
-- Process Loot/Items and fill loot log

-- add item from containerlink into the log and set state in Loot.actionLoot
-- state can be:
--   NONE  not decided what to do, let user decide to white/black list
--   DONT  not to handle, let user decide to white/black list
--   WHITE is not handled
--   BLACK/HANDLE item is handled
-- in case of no state: item is not shown, so button function has only to act on
-- keys/links in Loot.actionLoot with state HANDLE + BLACK

--  a) only on success of GetItemInfo()
--  b) if containerlink is identical to itemLink from GetItemInfo()
--  c) you never ever want handle quest items and wow tokens
--  d) if a same item was already judged, stay with the action
--  e) handle item class/subclass junk, only grays, because lockboxes are junk too ;-)
--  f) set gray items to be handled and ignore filtered items
--  g) don't handle items which have not sell value, they have to be blacklisted to delete if wanted
--  h) set items state if state was given as argument (if e.g. from bag filter)
--  i) if item was present at login and it was not blacklisted, set it to be not handled
--  j) if item is white or black listed set it's state
--  k) if still undecided and if its consumable, waepon, armor or recipe, set state to be handled
--  l) find it's market value and set market value to sell value on gray if enabled
--  m) compare market value with treshold and set state to be not handled if above
--  n) same for multiple items
--  o) correct market value and sell value for multiple items

function Loot:AddItem(list, link, item, state)
	Loot:DebugPrintf("AddItem(%s, %s, %s, %s)", tostring(list), tostring(link), tostring(item), tostring(state))

	local itemCount = item.itemCount or 1

--   a) only on success of GetItemInfo()
	local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, _,
		_, _, itemSellPrice, itemClassID, itemSubClassID, bindType, _, _,
		_ = addon.Item:GetItemInfo(link)

	if not itemName or not itemLink then
		Loot:DebugPrintf("  <<AddItem(): GetItemInfo(%s) was nil", tostring(link))
		return
	end

--   b) if containerlink is identical to itemLink from GetItemInfo()
	if link == itemLink then
		Loot:DebugPrintf("  rarity=%s level=%s minLevel=%s type=%s subtype=%s bind=%s",
			tostring(itemRarity), tostring(itemLevel), tostring(itemMinLevel), tostring(itemType), tostring(itemSubType), tostring(bindType))
	else
		Loot:DebugPrintf("ERR~  link different from itemLink, link = %s, itemLink = %s",
			addon.Item:EscapedLink(link),
			addon.Item:EscapedLink(itemLink))
		return
	end

--  c) you never ever want handle quest items and wow tokens
	if itemClassID == LE_ITEM_CLASS_WOW_TOKEN
		or itemClassID == LE_ITEM_CLASS_QUESTITEM
		or itemClassID == LE_ITEM_CLASS_REAGENT then
		return
	end

--  d) if a same item was already judged, stay with the action
	if Loot.actionLoot[link] then
		Loot:DebugPrintf("  has already state %s", tostring(Loot.STATE[Loot.actionLoot[link].state]))
	end

--  e) handle item class/subclass junk, only grays, because lockboxes are junk too ;-)
	if not itemRarity then
		Loot:DebugPrintf("  has no itemRarity: do nothing!")
		return
	end
	if not Loot.actionLoot[link]
		and itemClassID == LE_ITEM_CLASS_MISCELLANEOUS
		and itemSubClassID == LE_ITEM_MISCELLANEOUS_JUNK
		and (itemRarity and itemRarity == Enum.ItemQuality.Poor)
		then
			Loot.actionLoot[link] = { itemLink = itemLink, state = Loot.HANDLE, item = item}
			Loot:DebugPrintf("  is Junk, state %s", tostring(Loot.STATE[Loot.actionLoot[link].state]))
	end


--  f) set gray items to be handled and ignore filtered items
	local blue = Enum.ItemQuality.Rare or Enum.ItemQuality.Superior or 3
	if not Loot.actionLoot[link] and itemRarity then
		if itemRarity == Enum.ItemQuality.Poor then
			Loot.actionLoot[link] = { itemLink = itemLink, state = Loot.HANDLE, item = item}
			Loot:DebugPrintf("  is gray, state %s", tostring(Loot.STATE[Loot.actionLoot[link].state]))
		elseif itemRarity > Enum.ItemQuality.Epic then
			Loot:DebugPrintf("  is > epic: do nothing!")
			return
		elseif itemRarity >= blue and addon.db.global.ignoreRares then
			Loot.actionLoot[link] = { itemLink = itemLink, state = Loot.IGNORED, item = item}
			Loot:DebugPrintf("  is >= rare and is ignored, state %s", tostring(Loot.STATE[Loot.actionLoot[link].state]))
		elseif itemRarity >= Enum.ItemQuality.Epic and addon.db.global.ignoreEpics then
			Loot.actionLoot[link] = { itemLink = itemLink, state = Loot.IGNORED, item = item}
			Loot:DebugPrintf("  is >= epic and is ignored, state %s", tostring(Loot.STATE[Loot.actionLoot[link].state]))
		else
			Loot.actionLoot[link] = { itemLink = itemLink, state = Loot.NONE, item = item}
		end
	end

	if addon.db.global.ignoreLvl40 and itemLevel and tonumber(itemLevel) and tonumber(itemLevel) >= 40 then
		Loot:DebugPrintf("  >= lvl 40: do nothing!")
		Loot.actionLoot[link] = { itemLink = itemLink, state = Loot.IGNORED, item = item}
		return
	end

	if addon.db.global.ignoreLvl50 and itemLevel and tonumber(itemLevel) and tonumber(itemLevel) >= 50 then
		Loot:DebugPrintf("  >= lvl 50: do nothing!")
		Loot.actionLoot[link] = { itemLink = itemLink, state = Loot.IGNORED, item = item}
		return
	end

	if addon.db.global.ignoreLvl60 and itemLevel and tonumber(itemLevel) and tonumber(itemLevel) >= 60 then
		Loot:DebugPrintf("  >= lvl 60: do nothing!")
		Loot.actionLoot[link] = { itemLink = itemLink, state = Loot.IGNORED, item = item}
		return
	end

	if addon.db.global.ignoreLvl70 and itemLevel and tonumber(itemLevel) and tonumber(itemLevel) >= 70 then
		Loot:DebugPrintf("  >= lvl 70: do nothing!")
		Loot.actionLoot[link] = { itemLink = itemLink, state = Loot.IGNORED, item = item}
		return
	end

	if addon.db.global.ignoreLvl80 and itemLevel and tonumber(itemLevel) and tonumber(itemLevel) >= 80 then
		Loot:DebugPrintf("  >= lvl 80: do nothing!")
		Loot.actionLoot[link] = { itemLink = itemLink, state = Loot.IGNORED, item = item}
		return
	end

	if addon.db.global.ignoreLvl90 and itemLevel and tonumber(itemLevel) and tonumber(itemLevel) >= 90 then
		Loot:DebugPrintf("  >= lvl 90: do nothing!")
		Loot.actionLoot[link] = { itemLink = itemLink, state = Loot.IGNORED, item = item}
		return
	end

	if addon.db.global.ignoreLvl100 and itemLevel and tonumber(itemLevel) and tonumber(itemLevel) >= 100 then
		Loot:DebugPrintf("  >= lvl 100: do nothing!")
		Loot.actionLoot[link] = { itemLink = itemLink, state = Loot.IGNORED, item = item}
		return
	end

--  g) don't handle items which have not sell value, they have to be blacklisted to delete if wanted
	if itemSellPrice and tonumber(itemSellPrice) == 0 then
		Loot.actionLoot[link].state = Loot.DONT
		Loot:DebugPrintf("  has no sell value, state %s", tostring(Loot.STATE[Loot.actionLoot[link].state]))
		itemSellPrice = nil
	end

--  h) set items state if state was given as argument (if e.g. from bag filter)
	if state then
		Loot.actionLoot[link].state = state
		Loot:DebugPrintf("  set to default state %s", tostring(Loot.STATE[Loot.actionLoot[link].state]))
	end

--  i) if item was present at login and it was not blacklisted, set it to be not handled
	local itemID = item.itemID
	if itemID then
		if Loot.loginLoot[itemID] then
			Loot.actionLoot[link].state = Loot.DONT
			Loot:DebugPrintf("  was not dropped this session, state %s", tostring(Loot.STATE[Loot.actionLoot[link].state]))
			if not addon.db.global.showLoginLoot then
				if Loot.listLoot[link] and Loot.listLoot[link].state and Loot.listLoot[link].state == Loot.BLACK then
					Loot:DebugPrintf("  but was listed, state %s", tostring(Loot.STATE[Loot.actionLoot[link].state]))
					Loot.loginLoot[link] = nil
				else
					return
				end
			end
		end
	else
		Loot.actionLoot[link].state = Loot.DONT
		Loot:DebugPrintf("ERR~  has no proper itemID, state %s", tostring(Loot.STATE[Loot.actionLoot[link].state]))
	end

--  j) if item is white or black listed set it's state
	if Loot.listLoot[link] and Loot.listLoot[link].state then
		Loot.actionLoot[link].state = Loot.listLoot[link].state
		Loot:DebugPrintf("  was listed, state %s", tostring(Loot.STATE[Loot.actionLoot[link].state]))
	end

--  k) if still undecided and if its consumable, waepon, armor or recipe, set state to be handled
	if Loot.actionLoot[link].state == Loot.NONE and
		(
		itemClassID == LE_ITEM_CLASS_CONSUMABLE
		or itemClassID == LE_ITEM_CLASS_WEAPON
		or itemClassID == LE_ITEM_CLASS_ARMOR
		or itemClassID == LE_ITEM_CLASS_RECIPE
		-- or addon.isDebug
		)
		then
		Loot.actionLoot[link].state = Loot.HANDLE
		Loot:DebugPrintf("  is 0/C+2/W+4/A+9/R, state %s (debug=%s)", tostring(Loot.STATE[Loot.actionLoot[link].state]), tostring(addon.isDebug))
	end

--  l) find it's market value and set market value to sell value on gray if enabled
	local marketValue = addon.Item:GetItemValue(link, addon.db.global.priceSource)
	if itemRarity and itemRarity == Enum.ItemQuality.Poor and addon.db.global.useVendorSellForGrays then
		marketValue = itemSellPrice
	end

--  m) compare market value with treshold and set state to be not handled if above
	if Loot.actionLoot[link].state == Loot.HANDLE then
		if addon.db.global.handleTreshold and tonumber(addon.db.global.handleTreshold) then
			if marketValue and marketValue > tonumber(addon.db.global.handleTreshold) then
				Loot.actionLoot[link].state = Loot.TRESH
				Loot:DebugPrintf("  market value above treshold, state %s", tostring(Loot.STATE[Loot.actionLoot[link].state]))
			end
		end
	end

--  n) same for multiple items
	if Loot.actionLoot[link].state == Loot.HANDLE then
		if addon.db.global.handleTreshold and tonumber(addon.db.global.handleTreshold) and itemCount then
			if marketValue and (itemCount * marketValue) > tonumber(addon.db.global.handleTreshold) then
				if itemRarity and itemRarity > Enum.ItemQuality.Poor then
					Loot.actionLoot[link].state = Loot.TRESH
					Loot:DebugPrintf("  itemCount * market value above treshold, state %s", tostring(Loot.STATE[Loot.actionLoot[link].state]))
				end
			end
		end
	end

--  o) correct market value and sell value for multiple items
	if marketValue and tonumber(marketValue) and tonumber(marketValue) > 0 and tonumber(itemCount) then
		marketValue = marketValue * itemCount
	end
	if itemSellPrice and tonumber(itemSellPrice) and tonumber(itemCount) then
		itemSellPrice = itemSellPrice * itemCount
	end

	-- handle Fortune Cards etc. (currently en + de only)
	if Loot.actionLoot[link].state == Loot.NONE and itemClassID == LE_ITEM_CLASS_TRADEGOODS then
		if link and (
			link:match("h.Glückskarte") or
			link:match("h.Karte des Blutes") or
			link:match("h.Blood Card") or
			link:match("h.Fortune Card")) then
			Loot.actionLoot[link].state = Loot.HANDLE
			marketValue = itemSellPrice
			Loot:DebugPrintf("  sell card, state %s", tostring(Loot.STATE[Loot.actionLoot[link].state]))
		end
	end

	if Loot.actionLoot[link].state == Loot.NONE then
		Loot:DebugPrintf("WARN~  state %s", tostring(Loot.STATE[Loot.actionLoot[link].state]))
	end

	-- set action column
	local action = ""
	if Loot.actionLoot[link].state == Loot.TRESH then
		action = L["-- $$ --"]
	end
	if Loot.actionLoot[link].state == Loot.WHITE then
		action = L["-- WL --"]
	end
	if Loot.actionLoot[link].state == Loot.IGNORED then
		action = L["-- IG --"]
	end
	if Loot.actionLoot[link].state == Loot.DONT then
		action = L["-- DN --"]
	end

	-- change label if at vendor
	if Loot.atVendor then
		if Loot.actionLoot[link].state == Loot.BLACK or
			Loot.actionLoot[link].state == Loot.HANDLE then
			if itemSellPrice then
				action = L["To Sell"]
			else
				action = L["(To Sell)"]
			end
		end
	else
		if Loot.actionLoot[link].state == Loot.BLACK or
			Loot.actionLoot[link].state == Loot.HANDLE then
				action = L["To Delete"]
		end
	end

	-- put info into log
	addon.GUI.DebugLog(list, "1=%s# 2=%s# 3=%s# 4=%s# 5=%s# 6=%s# 7=%s# 8=%s#",
		tostring(itemCount), tostring(itemClassID), tostring(itemSubClassID), tostring(link),
		tostring(itemSellPrice), tostring(action), tostring(Loot.actionLoot[link].state), tostring(marketValue))
	Loot:DebugPrintf("  1=%s# 2=%s# 3=%s# 4=%s# 5=%s# 6=%s# 7=%s# 8=%s#",
		tostring(itemCount), tostring(itemClassID), tostring(itemSubClassID), tostring(link),
		tostring(itemSellPrice), tostring(action), tostring(Loot.actionLoot[link].state), tostring(marketValue))

	Loot:DebugPrintf("  <<AddItem for %s: %s", tostring(list), tostring(link))
end

-- Create Log with all loot in bags (minus loginLoot)
function Loot:FillLogFromBags(tab)
	Loot:DebugPrintf("FillLogFromBags(%s)", tostring(tab))

	-- check for removed items since Login
	local loot, iLoot, err = Loot:ScanLootInBagsAndInventory()

	if err and err ~= "" then
		Loot:DebugPrintf("ERR~ Error searching bags: %s", tostring(err))
		addon:Printf("Error searching %s", tostring(err))
	else
		Loot.loginLoot = Loot:RemoveLostLootFromLoginLoot(Loot.loginLoot, iLoot)

		-- reset all actions
		Loot.actionLoot = {}

		if tab then
			addon.GUI.DebugLog_Reset(tab)

			for link, item in pairs(loot) do
				if item.bag and link:match("Hitem:([0-9]+)") then
					local state
					if addon.db.global.ignoreBag4 and item.bag == 4 then
						state = Loot.IGNORED
					elseif addon.db.global.ignoreBag3 and item.bag == 3 then
						state = Loot.IGNORED
					elseif addon.db.global.ignoreBag2 and item.bag == 2 then
						state = Loot.IGNORED
					elseif addon.db.global.ignoreBag1 and item.bag == 1 then
						state = Loot.IGNORED
					elseif addon.db.global.ignoreBag0 and item.bag == 0 then
						state = Loot.IGNORED
					end
					Loot:AddItem(tab, link, item, state)
				end
			end
		end
	end
end

-- scan loot in bags and inventory
-- (return itemID based table too, as link from GetContainerItemInfo is sometimes shortened)
function Loot:ScanLootInBagsAndInventory()
	Loot:DebugPrintf("ScanLootInBagsAndInventory()")

	-- reset on Login()
	local loot = {}
	local iLoot = {}
	local err = ""

	for i = 0, NUM_BAG_SLOTS do
		for j = 1, GetContainerNumSlots(i) do
			local _, itemCount, locked, quality, readable, lootable, itemLink, _, noValue, _ = GetContainerItemInfo(i,j)
			if itemLink and itemCount and itemLink:match("Hitem:([0-9]+)") then
				local item = {
					itemCount = itemCount,
					locked = locked,
					quality = quality,
					readable = readable,
					lootable = lootable,
					itemLink = itemLink,
					noValue = noValue,
					itemID = addon.Item:GetItemID(itemLink),
					bag = i,
					}

				if item.itemID then
					Loot:DebugPrintf("  bag %s: add %s = %s as %s", tostring(item.bag), tostring(item.itemLink),
						addon.Item:EscapedLink(item.itemLink), tostring(item.itemID))
					loot[itemLink] = item
					iLoot[item.itemID] = item
				else
					err = err .. format("Bag %s/%s,", tostring(i), tostring(j))
					Loot:DebugPrintf("  bag %s: can't add %s = %s", tostring(item.bag), tostring(item.itemLink),
						addon.Item:EscapedLink(item.itemLink))
				end
			end
		end
	end

	for i = 1, 18 do
		local link = GetInventoryItemLink("player", i)
		if link and link:match("Hitem:([0-9]+)") then
			local item = {
				itemCount = 1,
				itemLink = link,
				itemID = addon.Item:GetItemID(link),
				slot = i,
				}
			if item.itemID then
				Loot:DebugPrintf("  inv slot %s: add %s = %s as %s", tostring(item.slot), tostring(item.itemLink),
					addon.Item:EscapedLink(item.itemLink), tostring(item.itemID))
				loot[link] = item
				iLoot[item.itemID] = item
			else
				err = err .. format("Inv %s,", tostring(i))
				Loot:DebugPrintf("  inv slot %s: dont add %s = %s", tostring(item.slot), tostring(item.itemLink),
					addon.Item:EscapedLink(item.itemLink))
			end
		end
	end

	return loot, iLoot, err
end

function Loot:RemoveLostLootFromLoginLoot(loginLoot, iLoot)
	Loot:DebugPrintf("RemoveLostLootFromLoginLoot(%s)", tostring(loginLoot))

	if loginLoot and iLoot then
		for itemID, _ in pairs(loginLoot) do
			if not iLoot[itemID] then
				Loot:DebugPrintf("  remove %s", tostring(itemID))
				loginLoot[itemID] = nil
			end
		end
	end

	return loginLoot
end

function Loot:SellOrDelete()
	Loot:DebugPrintf("SellOrDelete()")

	local sellCnt = 0

	for _, loot in pairs(Loot.actionLoot) do
		if loot.state == Loot.BLACK or
			loot.state == Loot.HANDLE then
			if Loot.atVendor then
				if sellCnt >= private.maxSellCnt then
					addon:Printf(L["Some items are left to sell. Press Sell again."])
					break
				end
				-- addon:Printf("Selling %s", tostring(loot.itemLink))
				sellCnt = Loot:SellItem(loot.itemLink, sellCnt)
			else
				-- addon:Printf("Deleting %s", tostring(loot.itemLink))
				sellCnt = Loot:DeleteItem(loot.itemLink, sellCnt)
			end
		end
	end

	if sellCnt >= 1 then
		if Loot.atVendor then
			Loot:SetStatusLine(format(L["Sold %s items."], tostring(sellCnt)), 3)
		else
			Loot:SetStatusLine(format(L["Deleted %s items."], tostring(sellCnt)), 3)
		end
	end

	if sellCnt > 1 then
		Loot:FillLogFromBags(addon.GUI.view)
		addon.GUI.DebugLog_UpdateData(addon.GUI.view, true)
	end
end

function Loot:SellItem(link, cnt)
	Loot:DebugPrintf("SellItem(%s, %s) - %s", tostring(link), tostring(cnt), addon.Item:EscapedLink(link))

	if not link then
		return cnt
	end

	local _, _, _, _, _, _, _, _, _, _, itemSellPrice = addon.Item:GetItemInfo(link)
	if itemSellPrice and itemSellPrice == 0 then
		addon:Printf(L["%s cannot be sold. Please try to delete it."], tostring(link))
		return cnt
	end

	local found
	local maxSold = false
	for bag = 0, NUM_BAG_SLOTS do
		if maxSold then
			break
		end
		for slot = 1, GetContainerNumSlots(bag) do
			local _, itemCount, locked, _, _, _, itemLink, _, _, _= GetContainerItemInfo(bag, slot)
			if itemLink and itemLink == link and not locked then
				if locked then
					addon:Printf("... %s is locked", tostring(link))
					Loot:DebugPrintf("... %s is locked", tostring(link))
				else
					if cnt >= private.maxSellCnt then
						addon:Printf(L["Some items are left to sell. Press Sell again."])
						maxSold = true
						break
					end

					-- sell!
					UseContainerItem(bag, slot)

					cnt = cnt + 1
					addon:Printf(L["%s.: %sx %s sold"], tostring(cnt), tostring(itemCount), tostring(link))
					found = true
				end
			end
		end
	end

	if not found then
		addon:Printf(L["didn't sell %s"], tostring(link))
	end
	return cnt
end

function Loot:DeleteItem(link, cnt)
	Loot:DebugPrintf("DeleteItem(%s) - %s", tostring(link), addon.Item:EscapedLink(link))

	if not link then
		return cnt
	end

	local found
	for bag = 0, NUM_BAG_SLOTS do
		for slot = 1, GetContainerNumSlots(bag) do
			local _, itemCount, locked, _, _, _, itemLink, _, _, _= GetContainerItemInfo(bag, slot)
			if itemLink and itemLink == link and not locked then
				if locked then
					addon:Printf("... %s is locked", tostring(link))
					Loot:DebugPrintf("... %s is locked", tostring(link))
				else
					-- delete!
					PickupContainerItem(bag, slot)
					DeleteCursorItem()

					cnt = cnt + 1
					addon:Printf(L["%s.: %sx %s deleted"], tostring(cnt), tostring(itemCount), tostring(link))
					found = true
				end
			end
		end
	end

	if not found then
		addon:Printf(L["didn't delete %s"], tostring(link))
	end
	return cnt
end

------------------------------------------------------------------------------
-- Sets self disappearing status text
function Loot:SetStatusLine(text, seconds)
	private.statusLine = text
	private.statusSeconds = seconds
end

-- Shows/Deletes status texts
function Loot:SecTimer()
	if private.statusLine and strlen(private.statusLine) > 0 then
		private.statusSeconds = private.statusSeconds or 1
		if private.statusFlag ~= nil and private.statusFlag >= private.statusSeconds then
			private.statusFlag = 0
			private.statusLine = ""
		else
			private.statusFlag = private.statusFlag or 0
			private.statusFlag = private.statusFlag + 1
		end
		addon.GUI:SetStatusLine(private.statusLine)
	end
	if private.bagUpdated then
		private.bagUpdated = false
		if addon.GUI.display then
			Loot:FillLogFromBags(addon.GUI.view)
			addon.GUI.DebugLog_UpdateData(addon.GUI.view, true)
		end
	end
end

-- EOF
