-------------------------------------------------------------------------
-- DeTrashLoot - Delete or sell items in bags based on their market value
-------------------------------------------------------------------------
-- Modules/Options.lua - Addon Options
--
-- Author: Expelliarm5s / June 2022 / All Rights Reserved
--
-- Version 1.1.7
-------------------------------------------------------------------------
-- luacheck: ignore 212 globals DLAPI
--------------------------------------

local addonName, addon = ...
local Options = addon:NewModule("Options", "AceConsole-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale(addonName)
--------------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------
-- Settings

Options.defaults = {
	profile = {
	},
	global = {
		timeFormat = "default",
		priceSource = "VendorSell",
		useVendorSellForGrays = true,
		handleTreshold = "5000000", --  500g
		maxVerbosity = "6",
		showExport = false,
		showLoginLoot = false,
		discardCopper = true,
		discardSilver = false,
		ignoreBag4 = false,
		ignoreBag3 = false,
		ignoreBag2 = false,
		ignoreBag1 = false,
		ignoreBag0 = false,
		ignoreRares = false,
		ignoreEpics = true,
		ignoreLvl40 = false,
		ignoreLvl50 = false,
		ignoreLvl60 = false,
		ignoreLvl70 = false,
		ignoreLvl80 = true,
		ignoreLvl90 = true,
		ignoreLvl100 = true,
		marketValueSort = true,
		textFontSize = "11",
		openWithVendor = false,
		closeWithVendor = false,
		extendedUI = false,
	},
}

Options.timeFormat = {}
Options.timeFormat["ago"]  = L["_ Hr. _ Min. ago"]
Options.timeFormat["%d.%m %H:%M:%S"] = "DD.MM HH:MM"
Options.timeFormat["%m/%d %H:%M:%S"] = "MM/DD HH:MM"
Options.timeFormat["%m-%d %H:%M:%S"] = "MM-DD HH:MM"

Options.textSizes = {}
Options.textSizes["10"]  = "10"
Options.textSizes["11"]  = "11"
Options.textSizes["12"]  = "12"
Options.textSizes["13"]  = "13"

------------------------------------------------------------------------------
-- Debug Stuff

function Options:DebugPrintf(...)
	if addon.isDebug then
		local status, res = pcall(format, ...)
		if status then
			addon:DebugLog("Opt~" .. res)
		end
	end
end

------------------------------------------------------------------------------
-- Addon Loading / Player Login/Logout

function Options.GetOptions(_, _, appName)
	if appName == addonName then

		local wowV, wowP = GetBuildInfo()
		local wowVersion = "|nGame: WoW, Flavor: " .. (addon.isClassic and "Classic" or "Retail") .. ", Version: " .. wowV .. ", Build: " .. wowP

		local blue = Enum.ItemQuality.Rare or Enum.ItemQuality.Superior or 3

		local options = {
			type = "group",
			name = addon.METADATA.NAME .. " (" .. addon.METADATA.VERSION .. ")",
			get = function(info)
					return addon.db.global[info[#info]] or ""
				end,
			set = function(info, value)
					addon.db.global[info[#info]] = value
					Options:DebugPrintf("OK~Set %s = %s", tostring(info[#info]), tostring(value))
					if addon.GUI.display then
						addon.Loot:FillLogFromBags(addon.GUI.view)
						addon.GUI.DebugLog_UpdateData(addon.GUI.view, true)
						-- addon.GUI.container:Reload()
					end
				end,
			args = {
				desc1a = {
					type = "description",
					order = 0,
					name = "|cff99ccff-: by " .. GetAddOnMetadata(addonName, "Author") .. " :-|r|n|n" .. GetAddOnMetadata(addonName, "Notes"),
					fontSize = "medium",
				},
				desc1b = {
					type = "description",
					order = 0.01,
					name = wowVersion,
				},
				header01a = {
					type = "header",
					name = L["Ignore loot to be sold or deleted"],
					order = 1.00,
				},
				desc1c = {
					type = "description",
					order = 1.01,
					name = L["Only consumables, weapons, armor, recipes, and junk can be sold or deleted.|n|n"] ..
						L["However, to prevent additional unwanted actions, you can ignore the contents of entire bags or rare or epic items:"],
					-- fontSize = "medium",
				},
				ignoreBag0 = {
					type = "toggle",
					name = L["Ignore ... Backpack"],
					desc = L["If checked, any loot in your backback is ignored."],
					order = 1.1,
					width = "normal",
					get = function(info) return addon.db.global[info[#info]] end,
				},
				ignoreBag1 = {
					type = "toggle",
					name = L["... Bag 1"],
					desc = L["If checked, any loot in the bag second from the right is ignored."],
					order = 1.2,
					width = "half",
					get = function(info) return addon.db.global[info[#info]] end,
				},
				ignoreBag2 = {
					type = "toggle",
					name = L["... Bag 2"],
					desc = L["If checked, any loot in the bag third from the left is ignored."],
					order = 1.3,
					width = "half",
					get = function(info) return addon.db.global[info[#info]] end,
				},
				ignoreBag3 = {
					type = "toggle",
					name = L["... Bag 3"],
					desc = L["If checked, any loot in the bag second from the left is ignored."],
					order = 1.31,
					width = "half",
					get = function(info) return addon.db.global[info[#info]] end,
				},
				ignoreBag4 = {
					type = "toggle",
					name = L["... Bag 4"],
					desc = L["If checked, any loot in the leftmost bag is ignored."],
					order = 1.32,
					width = "half",
					get = function(info) return addon.db.global[info[#info]] end,
				},
				ignoreRares = {
					type = "toggle",
					name = L["Ignore "] .. ITEM_QUALITY_COLORS[blue].hex .. L["Rares"] .. FONT_COLOR_CODE_CLOSE,
					desc = L["If checked, any loot with quality rare is ignored."],
					order = 1.33,
					width = "normal",
					get = function(info) return addon.db.global[info[#info]] end,
				},
				ignoreEpics = {
					type = "toggle",
					name = L["Ignore "] .. ITEM_QUALITY_COLORS[Enum.ItemQuality.Epic].hex .. L["Epics"] .. FONT_COLOR_CODE_CLOSE,
					desc = L["If checked, any loot with quality epic is ignored."],
					order = 1.34,
					width = "normal",
					get = function(info) return addon.db.global[info[#info]] end,
				},
				header01b = {
					type = "header",
					name = L["Do not show items with item level ..."],
					order = 1.35,
				},
				ignoreLvl40 = {
					type = "toggle",
					name = L[">= 40"],
					desc = L["If checked, any loot with item level 40 or higher is not shown in item lists."],
					order = 1.44,
					width = "half",
					get = function(info) return addon.db.global[info[#info]] end,
				},
				ignoreLvl50 = {
					type = "toggle",
					name = L[">= 50"],
					desc = L["If checked, any loot with item level 50 or higher is not shown in item lists."],
					order = 1.45,
					width = "half",
					get = function(info) return addon.db.global[info[#info]] end,
				},
				ignoreLvl60 = {
					type = "toggle",
					name = L[">= 60"],
					desc = L["If checked, any loot with item level 60 or higher is not shown in item lists."],
					order = 1.46,
					width = "half",
					get = function(info) return addon.db.global[info[#info]] end,
				},
				ignoreLvl70 = {
					type = "toggle",
					name = L[">= 70"],
					desc = L["If checked, any loot with item level 70 or higher is not shown in item lists."],
					order = 1.47,
					width = "half",
					get = function(info) return addon.db.global[info[#info]] end,
				},
				ignoreLvl80 = {
					type = "toggle",
					name = L[">= 80"],
					desc = L["If checked, any loot with item level 80 or higher is not shown in item lists."],
					order = 1.48,
					width = "half",
					get = function(info) return addon.db.global[info[#info]] end,
				},
				ignoreLvl90 = {
					type = "toggle",
					name = L[">= 90"],
					desc = L["If checked, any loot with item level 90 or higher is not shown in item lists."],
					order = 1.49,
					width = "half",
					get = function(info) return addon.db.global[info[#info]] end,
				},
				ignoreLvl100 = {
					type = "toggle",
					name = L[">= 100"],
					desc = L["If checked, any loot with item level 100 or higher is not shown in item lists."],
					order = 1.499,
					width = "half",
					get = function(info) return addon.db.global[info[#info]] end,
				},

				header02 = {
					type = "header",
					name = L["Market Value Settings"],
					order = 2.00,
				},
				useVendorSellForGrays = {
					type = "toggle",
					name = L["Use VendorSell for Grays"],
					desc = L["If checked, the price source VendorSell is used for gray items instead of the market value, which is really a wrong value for grays."],
					order = 2.1,
					width = "double",
					get = function(info) return addon.db.global[info[#info]] end,
				},
				marketValueSort = {
					type = "toggle",
					name = L["Sort by MarketValue"],
					desc = L["If checked, table is sorted by MarketValue."],
					order = 2.2,
					width = "normal",
					get = function(info) return addon.db.global[info[#info]] end,
					set = function(info, value)
						addon.db.global[info[#info]] = value
						Options:DebugPrintf("OK~Set %s = %s", tostring(info[#info]), tostring(value))
						if value then
							addon.GUI.debugLog_defaultSortTitle = -6
						else
							addon.GUI.debugLog_defaultSortTitle = 1
						end
						if addon.GUI.display then
							if addon.GUI.view and addon.GUI.debugLog_lastSortTitle[addon.GUI.view] then
								addon.GUI.debugLog_lastSortTitle[addon.GUI.view] = nil
							end
							addon.GUI.container:Reload()
						end
					end
				},
				discardSilver = {
					type = "toggle",
					name = L["Reduce space of ... silver values"],
					desc = L["If checked, any values below 1g are not displayed."],
					order = 2.3,
					width = "double",
					get = function(info) return addon.db.global[info[#info]] end,
				},
				discardCopper = {
					type = "toggle",
					name = L["... copper values"],
					desc = L["If checked, any values below 1s are not displayed."],
					order = 2.4,
					width = "normal",
					get = function(info) return addon.db.global[info[#info]] end,
				},

				header03 = {
					type = "header",
					name = L["Misc."],
					order = 3.0,
				},
				openWithVendor = {
					type = "toggle",
					name = L["Open UI at Vendor"],
					desc = L["If checked, UI is opened when talking to a vendor."],
					order = 3.1,
					width = "double",
					get = function(info) return addon.db.global[info[#info]] end,
				},
				closeWithVendor = {
					type = "toggle",
					name = L["Close UI at Vendor"],
					desc = L["If checked, UI is closed after talking to a vendor."],
					order = 3.2,
					width = "double",
					get = function(info) return addon.db.global[info[#info]] end,
				},
				extendedUI = {
					type = "toggle",
					name = L["Show Item Class and SubClass"],
					desc = L["If checked, UI shows the item class and subclass column."],
					order = 3.3,
					width = "double",
					get = function(info) return addon.db.global[info[#info]] end,
					set = function(info, value)
						addon.db.global[info[#info]] = value
						Options:DebugPrintf("OK~Set %s = %s", tostring(info[#info]), tostring(value))
						addon.Loot:SetRebuildUI()
						if addon.GUI.display then
							addon.GUI.display:Fire("OnClose")
							addon.Loot:Load(addon.GUI.view)
						end
					end
				},
				desc1d = {
					type = "description",
					order = 3.35,
					name = " ",
					fontSize = "medium",
				},
				desc1e = {
					type = "description",
					order = 3.4,
					name = L["|cffff8888Changing the font size causes an user interface reload!|r"],
					fontSize = "medium",
				},
				textFontSize = {
					type = "select",
					style = "dropdown",
					name = L["Item List Font Size"],
					desc = L["Choose the font size. Changing the size causes an user interface reload!"],
					order = 3.5,
					width = "double",
					values = Options.textSizes,
					set = function(info, value)
						addon.db.global[info[#info]] = value
						Options:DebugPrintf("OK~Set %s = %s", tostring(info[#info]), tostring(value))
						ReloadUI()
					end
				},

--				header04 = {
--					type = "header",
--					name = L["DANGEROUS SETTINGS"],
--					order = 4.00,
--				},
--				showLoginLoot = {
--					type = "toggle",
--					name = L["Show Loot Present at Login/Reload (BE CAREFUL!)"],
--					desc = L["If checked, any loot present at login/reload is shown in the loot list. BE CAREFUL WHEN BLACKLIST ITEMS!!!"],
--					order = 4.1,
--					width = "double",
--					get = function(info) return addon.db.global[info[#info]] end,
--				},
			},
		}

		return options
	end
end

-- EOF
